unit phScrollBox;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, phUtil;

type
  TphScrollBarKind = (phsbHorizontal, phsbVertical);

function InitializeFlatSB(hWnd: HWND): integer; stdcall;
function UninitializeFlatSB(hWnd: HWND): integer; stdcall;
function FlatSB_ShowScrollBar(hWnd: HWND; SBKind: TphScrollBarKind;
	Show: Boolean): integer; stdcall;
function FlatSB_EnableScrollBar(hWnd: HWND; SBKind: TphScrollBarKind;
	Value: integer): integer; stdcall;
function FlatSB_GetScrollInfo(hWnd: HWND; BarFlag: Integer;
	var ScrollInfo: TScrollInfo): integer; stdcall;    
function FlatSB_SetScrollInfo(hWnd: HWND; BarFlag: Integer;
	var ScrollInfo: TScrollInfo; Redraw: Boolean): integer; stdcall;


type
  //TphScrollBarKind = (phsbHorizontal, phsbVertical);
  TphScrollEvent = procedure (Sender: TObject; Kind: TphScrollBarKind) of Object;

	TphScrollBox = class;

	TphScrollBar = class(TPersistent)
  private
  	FControl: TphScrollBox;
    FKind: TphScrollBarKind;
    FMin: integer;
    FMax: integer;
    FPage: integer;
    FPosition: integer;
    FTrackPos: integer;
    FLargeChange: integer;
    FSmallChange: integer;
    FThumbValue: integer;
    FVisible: Boolean;
    FScrollInfo: TScrollInfo;

    procedure SetMin(Value: integer);
    procedure SetMax(Value: integer);
    procedure SetSmallChange(Value: integer);
    procedure SetLargeChange(Value: integer);
    procedure SetPage(Value: integer);
    procedure SetPosition(Value: integer);
    function GetTrackPos: integer;
    procedure SetScrollData(Value: integer; Mask: Cardinal; MinMax: Boolean);
    function GetScrollData(Mask: Cardinal): integer;
    procedure ScrollMessage(var Msg: TWMScroll);
    constructor Create(AControl: TphScrollBox; AKind: TphScrollBarKind);
    procedure SetVisible(Value: Boolean);
    function GetBarKind: Word;
  protected

  public
    procedure Assign(Source: TPersistent); override;
    property Kind: TphScrollBarKind read FKind;
    property TrackPos: integer read GetTrackPos;
  published
  	property Min: integer read FMin write SetMin;
    property Max: integer read FMax write SetMax;
    property SmallChange: integer
    	read FSmallChange write SetSmallChange;
    property LargeChange: integer
    	read FLargeChange write SetLargeChange;
    property Position: integer read FPosition write SetPosition;

    property ThumbValue: integer read FThumbValue write FThumbValue;
    property Visible: Boolean read FVisible write SetVisible;

  end;

  TphScrollBox = class(TWinControl)
  private
  	FVersion: Boolean;
  	FBorderStyle: TBorderStyle;
    FHorzScrollBar: TphScrollBar;
    FVertScrollBar: TphScrollBar;
    FFlat: Boolean;
    FOnResize: TNotifyEvent;
    FOnChange: TphScrollEvent;
    procedure WMHScroll(var Message: TWMHScroll); message WM_HSCROLL;
    procedure WMVSCroll(var Message: TWMVScroll); message WM_VSCROLL;
    procedure WMSize(var Message: TWMSize); message WM_SIZE;
    procedure WMNCHitTest(var Message: TMessage); message WM_NCHITTEST;
    procedure CMCtl3DChanged(var Message: TMessage); message CM_CTL3DCHANGED;
    procedure SetHorzScrollBar(Value: TphScrollBar);
    procedure SetVertScrollBar(Value: TphScrollBar);
    procedure SetBorderStyle(Value: TBorderStyle);
    procedure SetFlat(Value: Boolean);
  protected
  	procedure CreateParams(var Params: TCreateParams); override;
    procedure CreateWnd; override;
    procedure Resize; dynamic;
  public
  	constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
   	property Align;
    property HorzScrollBar: TphScrollBar
    	read FHorzScrollBar
      write SetHorzScrollBar;
    property VertScrollBar: TphScrollBar
    	read FVertScrollBar
      write SetVertScrollBar;
    property BorderStyle: TBorderStyle
    	read FBorderStyle
      write SetBorderStyle default bsSingle;
    property Flat: Boolean
    	read FFlat
      write SetFlat;
    property Color nodefault;
    property Ctl3D;
    property Font;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize: TNotifyEvent read FOnResize write FOnResize;
    property OnChange: TphScrollEvent
    	read FOnChange
      write FOnChange;
  end;

procedure Register;

implementation

const
  cctrl = 'comctl32.dll';

function InitializeFlatSB; external cctrl name 'InitializeFlatSB';
function UninitializeFlatSB; external cctrl name 'UninitializeFlatSB';
function FlatSB_ShowScrollBar; external cctrl name 'FlatSB_ShowScrollBar';
function FlatSB_EnableScrollBar; external cctrl name 'FlatSB_EnableScrollBar';
function FlatSB_GetScrollInfo; external cctrl name 'FlatSB_GetScrollInfo';
function FlatSB_SetScrollInfo; external cctrl name 'FlatSB_SetScrollInfo';

constructor TphScrollBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  FHorzScrollBar := TphScrollBar.Create(Self, phsbHorizontal);
  FVertScrollBar := TphScrollBar.Create(Self, phsbVertical);

  ControlStyle := [csAcceptsControls, csCaptureMouse, csClickEvents,
    csSetCaption, csDoubleClicks];

  //check the comctl32.dll version first
  //if not version 4.71 or higher, only normal scrollbar can ba used
  FVersion:= CheckComCtlVersion;

  Width := 185;
  Height := 41;
  FBorderStyle := bsSingle;

  //Default use normal scrollbar
  FFlat:= False;

end;

procedure TphScrollBox.CreateParams(var Params: TCreateParams);
const
  BorderStyles: array[TBorderStyle] of Longint = (0, WS_BORDER);
begin
  inherited CreateParams(Params);
  with Params do
  begin
    Style := Style or BorderStyles[FBorderStyle];
    //WindowClass.style := WindowClass.style or CS_HREDRAW or CS_VREDRAW;
    if NewStyleControls and Ctl3D and (FBorderStyle = bsSingle) then
    begin
      Style := Style and not WS_BORDER;
      if FVersion = True then
      begin
      	if FFlat = True then
      		//for better visual effect if use flat scrollbar
      		ExStyle := ExStyle or WS_EX_STATICEDGE
      	else
      		ExStyle:= ExStyle or WS_EX_CLIENTEDGE;
      end
      else
      	ExStyle:= ExStyle or WS_EX_CLIENTEDGE;
    end;
  end;
end;

procedure TphScrollBox.CreateWnd;
begin
  inherited CreateWnd;

  if Fversion = True then
  begin
  	if FFlat = true then
  		InitializeFlatSB(handle)
  	else
  		UninitializeFlatSB(handle);

  	if FHorzScrollBar.Visible = true then
  		FlatSB_ShowScrollBar(Handle, phsbHorizontal, True);

  	if FVertScrollBar.Visible = true then
  		FlatSB_ShowScrollBar(Handle, phsbVertical, True);
  end
  else
  begin
  	if FHorzScrollBar.Visible = true then
  		ShowScrollBar(Handle, SB_HORZ, True);

  	if FVertScrollBar.Visible = true then
  		ShowScrollBar(Handle, SB_VERT, True);
  end;
end;



destructor TphSCrollBox.Destroy;
begin
  FHorzScrollBar.Free;
  FVertScrollBar.Free;
  
	inherited Destroy;
end;

procedure TphScrollBox.WMHScroll(var Message: TWMHScroll);
begin
  if (Message.ScrollBar = 0) and FHorzScrollBar.Visible then
		FHorzScrollBar.ScrollMessage(Message)
  else
  	inherited;
end;

procedure TphScrollBox.WMVSCroll(var Message: TWMVScroll);
begin

  if (Message.ScrollBar = 0) and FVertScrollBar.Visible then
		FVertScrollBar.ScrollMessage(Message)
  else
  	inherited;
end;

procedure TphScrollBox.SetHorzScrollBar(Value: TphScrollBar);
begin
  FHorzScrollBar.Assign(Value);
end;

procedure TphScrollBox.SetVertScrollBar(Value: TphScrollBar);
begin
  FVertScrollBar.Assign(Value);
end;

procedure TphScrollBox.SetBorderStyle(Value: TBorderStyle);
begin
  if Value <> FBorderStyle then
  begin
    FBorderStyle := Value;
    RecreateWnd;
  end;
end;

procedure TphScrollBox.SetFlat(Value: Boolean);
begin
	if FVersion = False then
  begin
  	MessageDlg('Flat ScrollBar cannot be used ' +
    	'the version of COMCTL32.dll in this system is lower than 4.71',
    	mtInformation, [mbOK], 0);
    exit;
  end;

	if FFlat <> Value then
  begin
  	FFlat:= Value;
    RecreateWnd;
  end;
end;

procedure TphScrollBox.Resize;
begin
  
  if Assigned(FOnResize) then FOnResize(Self);
end;

procedure TphScrollBox.WMSize(var Message: TWMSize);
begin
  inherited;
  if not (csLoading in ComponentState) then Resize;


end;

procedure TphScrollBox.WMNCHitTest(var Message: TMessage);
begin
  DefaultHandler(Message);
end;

procedure TphScrollBox.CMCtl3DChanged(var Message: TMessage);
begin
  if NewStyleControls and (FBorderStyle = bsSingle) then RecreateWnd;
  inherited;
end;

//********************* TphScrollBar ***************************//
constructor TphScrollBar.Create(AControl: TphScrollBox; AKind: TphScrollBarKind);
begin
  inherited Create;

  FControl:= AControl;
  FKind:= AKind;
  FVisible:= True;
  
  FSmallChange:= 1;
  FLargeChange:= 10;

  FMin:= 0;
  FMax:= 100;
  FPage:= 1;
  FPosition:= 0;
  FTrackPos:= 0;
  FThumbValue:= 1;

  FScrollInfo.cbSize:= sizeof(TScrollInfo);
  FScrollInfo.nMin:= FMin;
  FScrollInfo.nMax:= FMax;
  FScrollInfo.nPage:= FPage;
  FScrollInfo.nPos:= FPosition;
  FScrollInfo.nTrackPos:= FTrackPos;

end;

procedure TphScrollBar.SetScrollData(Value: integer; Mask: Cardinal; MinMax: Boolean);
begin
	FScrollInfo.fMask:= Mask;

  if (Mask = SIF_RANGE) and (MinMax = true) then
    FScrollInfo.nMin:= Value
  else if (Mask = SIF_RANGE) and (MinMax = false) then
    FScrollInfo.nMax:= Value
  else if Mask = SIF_PAGE then
    FScrollInfo.nPage:= Value
  else if Mask = SIF_POS then
  	FScrollInfo.nPos:= Value;

  if FControl.FVersion = True then
  	FlatSB_SetScrollInfo(FControl.Handle, GetBarKind, FScrollInfo, True)
	else
  	SetScrollInfo(FControl.Handle, GetBarKind, FScrollInfo, True);


end;

function TphScrollBar.GetScrollData(Mask: Cardinal): integer;
begin
  FScrollInfo.fMask:= Mask;

  if FControl.FVersion = True then
		FlatSB_GetScrollInfo(FControl.Handle, GetBarKind, FScrollInfo)
  else
  	GetScrollInfo(FControl.Handle, GetBarKind, FScrollInfo);

  result:= FScrollInfo.nTrackPos;

end;

procedure TphScrollBar.SetMin(Value: integer);
begin
	if FMin <> Value then
  begin
  	FMin:= Value;
  	SetScrollData(Value, SIF_RANGE, True);
  end;
end;

procedure TphScrollBar.SetMax(Value: integer);
begin
	if FMax <> Value then
  begin
  	FMax:= Value;
  	SetScrollData(Value, SIF_RANGE, False);
  end;
end;

procedure TphScrollBar.SetPage(Value: integer);
begin
	if FPage <> Value then
  begin
  	FPage:= Value;
  	SetScrollData(Value, SIF_PAGE, True);
  end;
end;

procedure TphScrollBar.SetPosition(Value: integer);
begin
	if FPosition <> Value then
  begin
  	FPosition:= Value;
  	SetScrollData(Value, SIF_POS, True);
  end;
end;

function TphScrollBar.GetTrackPos: integer;
begin
  FTrackPos:= GetScrollData(SIF_ALL);
	result:= FTrackPos;
end;

procedure TphScrollBar.SetSmallChange(Value: integer);
begin
	if Value <> FSmallChange then
  	FSmallChange:= Value;
end;

procedure TphScrollBar.SetLargeChange(Value: integer);
begin
	if Value <> FLargeChange then
  begin
		FLargeChange:= Value;

    //Page value of scrollbar is the same as LargeChage
    //update the page value of scrollbar here
    SetPage(Value);
  end;
end;

procedure TphScrollBar.Assign(Source: TPersistent);
begin
	if Source is TphScrollBar then
	begin
    Visible := TphScrollBar(Source).Visible;
    Min:= TphScrollBar(Source).Min;
    Max:= TphScrollBar(Source).Max;
    SmallChange:= TphScrollBar(Source).SmallChange;
    LargeChange:= TphScrollBar(Source).LargeChange;
    Position:= TphScrollBar(Source).Position;
    ThumbValue:= TphScrollBar(Source).ThumbValue;
    Exit;
  end;
  inherited Assign(Source);
end;

procedure TphScrollBar.ScrollMessage(var Msg: TWMScroll);
var
	Value: Integer;

begin

  Value:= 0;
	with Msg do
  begin
    case ScrollCode of
      SB_LINELEFT: Value:= FPosition - FSmallChange;
      SB_LINERIGHT: Value:= FPosition + FSmallChange;
      SB_PAGELEFT: Value:= FPosition - FLargeChange;
      SB_PAGERIGHT: Value:= FPosition + FLargeChange;
      SB_THUMBPOSITION: Value:= TrackPos - TrackPos Mod ThumbValue;
      SB_THUMBTRACK: Value:= TrackPos - TrackPos Mod ThumbValue;
      SB_TOP: Value:= FMin;
      SB_BOTTOM: Value:= FMax;
      SB_ENDSCROLL: begin exit; end;
    end;

  end;

  if Value <= FMin then
    Value:= FMin;

  if Value >= FMax then
  	Value:= FMax;

  SetPosition(Value);
  if Assigned (FControl.FOnChange) then FControl.FOnChange(self, FKind);
end;

procedure TphScrollBar.SetVisible(Value: Boolean);
var
  ScrollInfo: TScrollInfo;
begin
		if FVisible <> Value then
    begin
		FVisible:= Value;

		ScrollInfo.fMask:= SIF_ALL;
		ScrollInfo.cbSize:= sizeof(TScrollInfo);
  	ScrollInfo.nMin:= 0;
    if FVisible = true then
  		ScrollInfo.nMax:= FMax
    else
    	SCrollInfo.nMax:= 0;

  	ScrollInfo.nPage:= FPage;
  	ScrollInfo.nPos:= FPosition;
  	ScrollInfo.nTrackPos:= FTrackPos;

    if FControl.FVersion = True then
    	FlatSB_SetScrollInfo(FControl.Handle, GetBarKind, ScrollInfo, True)
    else
    	SetScrollInfo(FControl.Handle, GetBarKind, ScrollInfo, True);


 	end;
end;

function TphScrollBar.GetBarKind: Word;
begin
	Result:= SB_HORZ;
  if Kind = phSBVertical then
  	Result:= SB_VERT;

end;

procedure Register;
begin
  RegisterComponents('phWare', [TphScrollBox]);
end;

end.
