unit phUtil;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, extctrls;

function CheckComctlVersion: Boolean;
function GetComCtlVersion(var Minor: Integer; var Major: Integer): boolean;
procedure GetComCtlInfo;

type
	PDllVersionInfo = ^TDllVersionInfo;
  TDllVersionInfo = record
  	Size: integer;
		MajorVersion: integer;
		MinorVersion: integer;
		BuildNumber: integer;
		PlatformID: integer;
  end;

	TphCaret = class (TObject)
  private
  	FCanvas: TCanvas;
    FLeft: Integer;
    FTop: Integer;
    FWidth: Integer;
    FHeight: Integer;
    FVisible: Boolean;
    FTimer: TTimer;
    FBlinkTime: Cardinal;
    FBlinkStatus: Boolean;
    procedure OnTimer(Sender: TObject);
    procedure SetValue(var Value1: integer; Value2: integer);
  protected
  	procedure SetWidth(Value: integer);
    procedure SetHeight(Value: integer);
  	procedure SetLeft(Value: Integer);
    procedure SetTop(Value: Integer);
    procedure SetVisible(Value: Boolean);
    procedure SetBlinkTime(Value: Cardinal);
    procedure UpdateCaret;
  public
  	constructor Create(PaintControl: TPaintBox);
    destructor destroy; override;
  published
  	property Width: integer
    	read FWidth
      write SetWidth;
    property Height: integer
    	read FHeight
      write SetHeight;
  	property Left: integer
    	read FLeft
      write SetLeft;
    property Top: integer
    	read FTop
      write SetTop;
    property Visible: Boolean
    	read FVisible
      write SetVisible;
    property BlinkTime: Cardinal
    	read FBlinkTime
      write SetBlinkTime;
  end;

implementation

var
	_GetDllVersionInfo: function(var dvInfo: TDllVersionInfo): integer; stdcall;

function CheckComctlVersion: Boolean;
var
	Minor, Major: integer;
begin
	Result:= GetComCtlVersion(Minor, Major);
  
end;

function GetComCtlVersion(var Minor: Integer; var Major: Integer): boolean;
var
	hInst: THandle;
  dvInfo: TDllVersionInfo;
  VerResult: Integer;
begin
	hInst:= LoadLibrary('comctl32.dll');

  if (hInst >= 0) and (hInst < 32) then
  begin
  	MessageDlg('Error loading comctl32.dll', mtError, [mbOK], 0);
  	Result:= False;
  end
  else
  begin
  	@_GetDllVersionInfo:= GetProcAddress(hInst, 'DllGetVersion');

    //only the new version of comctl32.dll has this function
    if @_GetDllVersionInfo <> nil then
    begin
    	result:= true;
    	dvInfo.Size:= sizeof(TDllVersionInfo);

    	verResult:= _GetDllVersionInfo(dvInfo);

      if verResult = 0 then
      begin
    		Minor:= dvInfo.MinorVersion;
    		Major:= dvInfo.MajorVersion;
      end
      else
      	result:= False;
    end
    else
    begin
    	result:= False;
    end;

  end;

  FreeLibrary(hInst);

end;

procedure GetComCtlInfo;
var
	Major, Minor: Integer;
  VerText, InfoText: string;
begin
	GetComCtlVersion(Minor, Major);

  VerText:= 'COMCTL32 Version ' + IntToStr(Major) + '.' +
  	IntToStr(Minor);

  case Major of
  	4:
    	begin
      	case Minor of
        	0: InfoText:= 'Distributed via Microsoft Windows 95/Windows NT 4.0';
          70:InfoText:= 'Distributed via Microsoft Internet Explorer 3.x';
          71:InfoText:= 'Distributed via Microsoft Internet Explorer 4.0';
          72:InfoText:= 'Distributed via Microsoft Internet Explorer 4.01/Windows 98';
         
        end;
      end;
  	5:
    	begin
      	case Minor of
        	0: InfoText:= 'Distributed via Microsoft Windows NT 5.0';

        end;
      end;
  end;

  MessageDlg(VerText + #13#10 + InfoText, mtInformation, [mbOK], 0);
  
end;

constructor TphCaret.Create(PaintControl: TPaintBox);
begin
	FCanvas:= PaintControl.Canvas;
  
  FLeft:= 0;
  FTop:= 0;
  FWidth:= 10;
  FHeight:= 10;
  FVisible:= True;

  FBlinkTime:= 500;
  FBlinkStatus:= False;
  FTimer:= TTimer.Create(PaintControl);
  FTimer.OnTimer:= OnTimer;
  FTimer.Interval:= FBlinkTime;
  FTimer.Enabled:= true;
end;

destructor TphCaret.Destroy;
begin
	FTimer.Enabled:= False;
  FTimer.Free;
	inherited Destroy;
end;

procedure TphCaret.OnTimer(Sender: TObject);
begin
	UpdateCaret;
  FBlinkStatus:= not FBlinkStatus;
end;

procedure TphCaret.SetValue(var Value1: integer; Value2: integer);
begin
	if Visible = true then
  begin
  	Visible:= False;
  	Value1:= Value2;
  	Visible:= True;
  end
  else
  	Value1:= Value2;
end;

procedure TphCaret.SetBlinkTime(Value: Cardinal);
begin
	if FBlinkTime <> Value then
  begin
  	FBlinkTime:= Value;
    FTimer.Interval:= Value;
  end;
end;

procedure TphCaret.SetWidth(Value: Integer);
begin
	if FWidth <> Value then
  begin
  	SetValue(FWidth, Value);
  end;
end;

procedure TphCaret.SetHeight(Value: Integer);
begin
	if FHeight <> Value then
  begin
  	SetValue(FHeight, Value);
  end;
end;

procedure TphCaret.SetLeft(Value: Integer);
begin
	if FLeft <> Value then
  begin
  	SetValue(FLeft, Value);
  end;
end;

procedure TphCaret.SetTop(Value: Integer);
begin
	if FTop <> Value then
  begin
  	SetValue(FTop, Value);
  end;
end;

procedure TphCaret.SetVisible(Value: Boolean);
begin
	if FVisible <> Value then
  begin
  	FVisible:= Value;
    if FVisible = True then
    	FTimer.Enabled:= True
    else
    	FTimer.Enabled:= False;

     //if Visible = false and  
    //if FBlinkStatus = true, it means caret just be drawn on canvas
    //Clear the caret on canvas
    if (FBlinkStatus = true) and (FVisible = False) then
    begin
    	//set FBlinkStatus value back to false, it means no draw on canvas
    	FBlinkStatus:= false;
    	UpdateCaret;
    end;
  end;
end;

procedure TphCaret.UpdateCaret;
begin                      
	PatBlt(FCanvas.Handle, FLeft, FTop, FWidth, FHeight, PATINVERT);

end;

end.
